@@ -16,7 +16,7 @@ HHD_PID_MOTION = 0x11
 HHD_PID_VENDOR = 0x7000
 
 CONTROLLER_THEMES: dict[str, tuple[int, int, str]] = {
-    "hhd": (0x5335, 0x0001, "Handheld Daemon Controller"),
+    "hhd": (0x045E, 0x028F, "Handheld Daemon Controller"),
     # Sony
     "ps3": (0x054C, 0x0268, "DualShock 3"),
     "ps4": (0x054C, 0x05C4, "DualShock 4"),
@@ -37,7 +37,7 @@ CONTROLLER_THEMES: dict[str, tuple[int, int, str]] = {
     "steam_deck": (0x28DE, 0x1205, "Steam Deck"),
     "steam_controller": (0x28DE, 0x1202, "Steam Controller"),
     "steam_input": (0x28DE, 0x11FF, "Steam Input"),
-    "hori_steam": (0x0f0d, 0x0196, "Steam Controller (HHD)")
+    "hori_steam": (0x0F0D, 0x0196, "Steam Controller (HHD)"),
 }
 
 
@@ -47,6 +47,15 @@ def B(b: str | Sequence[str], num: int | None = None):
     assert b, f"No value provided."
     if not isinstance(b, str):
         b = b[0]
+
+    try:
+        # .ecodes misses UInput stuff, grab from runtime if it exists
+        import evdev.ecodes_runtime as ecodes_runtime
+
+        return cast(int, getattr(ecodes_runtime, b))
+    except Exception:
+        pass
+
     return cast(int, getattr(evdev.ecodes, b))
 
 
