@@ -4,17 +4,19 @@ import select
 import time
 from threading import Event as TEvent
 
-
-from hhd.controller import Multiplexer, DEBUG_MODE
+from hhd.controller import DEBUG_MODE, Multiplexer
+from hhd.controller.lib.hide import unhide_all
 from hhd.controller.physical.evdev import B as EC
 from hhd.controller.physical.evdev import GenericGamepadEvdev, enumerate_evs
+from hhd.controller.physical.hidraw import enumerate_unique
 from hhd.controller.physical.imu import CombinedImu, HrtimerTrigger
 from hhd.controller.virtual.uinput import UInputDevice
 from hhd.plugins import Config, Context, Emitter, get_gyro_state, get_outputs
-from .serial import SerialDevice
+
+from .const import BTN_MAPPINGS, BTN_MAPPINGS_NONTURBO, DEFAULT_MAPPINGS
 from .hid_v1 import OxpHidraw
 from .hid_v2 import OxpHidrawV2
-from .const import BTN_MAPPINGS, DEFAULT_MAPPINGS, BTN_MAPPINGS_NONTURBO
+from .serial import SerialDevice, get_serial
 
 FIND_DELAY = 0.1
 ERROR_DELAY = 0.3
@@ -42,7 +44,7 @@ XFLY_PID = 0xB001
 XFLY_PAGE = 0xFF01
 XFLY_USAGE = 0x0001
 
-BACK_BUTTON_DELAY = 0.1
+BUTTON_MIN_DELAY = 0.13
 
 RGB_MODES_FULL = {
     "disabled": [],
@@ -55,6 +57,11 @@ RGB_MODES_STICKS = {
     "oxp": ["oxp"],
     "solid": ["color"],
 }
+RGB_MODES_STICKS_AOK = {
+    "disabled": [],
+    "aok": ["aok"],
+    "solid": ["color"],
+}
 
 
 def plugin_run(
@@ -70,40 +77,99 @@ def plugin_run(
     init = time.perf_counter()
     repeated_fail = False
     switch_to_turbo = None
+    first_disabled = True
 
     while not should_exit.is_set():
         if conf["controller_mode.mode"].to(str) == "disabled":
-            # Close the volume keyboard cache
-            UInputDevice.close_volume_cached()
             time.sleep(ERROR_DELAY)
+            if first_disabled:
+                UInputDevice.close_volume_cached()
+                unhide_all()
+            first_disabled = False
             continue
+        else:
+            first_disabled = True
 
         try:
-            found_device = bool(enumerate_evs(vid=GAMEPAD_VID))
+            found_device = bool(enumerate_evs(vid=GAMEPAD_VID, pid=GAMEPAD_PID))
         except Exception:
             logger.warning("Failed finding device, skipping check.")
             time.sleep(LONGER_ERROR_DELAY)
             found_device = True
 
-        if not found_device:
+        try:
+            protocol = dconf.get("protocol", None)
+            # Serial device is always present
+            # Hid devices might not be, wait a bit for them
+            match protocol:
+                case "hid_v1":
+                    found_vendor = bool(
+                        enumerate_unique(
+                            vid=X1_MINI_VID,
+                            pid=X1_MINI_PID,
+                            usage_page=X1_MINI_PAGE,
+                            usage=X1_MINI_USAGE,
+                        )
+                    )
+                case "hid_v2":
+                    found_vendor = bool(
+                        enumerate_unique(
+                            vid=XFLY_VID,
+                            pid=XFLY_PID,
+                            usage_page=XFLY_PAGE,
+                            usage=XFLY_USAGE,
+                        )
+                    )
+                case "hid_v1_g1":
+                    found_vendor = bool(
+                        enumerate_unique(
+                            vid=XFLY_VID,
+                            pid=XFLY_PID,
+                            usage_page=XFLY_PAGE,
+                            usage=XFLY_USAGE,
+                        )
+                    )
+                case "mixed":
+                    found_vendor = bool(
+                        enumerate_unique(
+                            vid=XFLY_VID,
+                            pid=XFLY_PID,
+                            usage_page=XFLY_PAGE,
+                            usage=XFLY_USAGE,
+                        )
+                    ) and bool(get_serial()[0])
+                case "serial":
+                    found_vendor = bool(get_serial()[0])
+                case _:
+                    found_vendor = True
+        except Exception:
+            logger.warning("Failed finding vendor device, skipping check.")
+            found_vendor = True
+
+        turbo_start = False
+        if not found_device or not found_vendor:
             curr = time.perf_counter()
             if first:
                 logger.info("Controller not found. Waiting...")
                 switch_to_turbo = curr + TURBO_DELAY
             time.sleep(FIND_DELAY)
             first = False
-            if turbo and switch_to_turbo and curr > switch_to_turbo:
+            if found_vendor and turbo and switch_to_turbo and curr > switch_to_turbo:
                 logger.info("Switching to turbo only button mode")
                 updated.clear()
-                turbo_loop(conf.copy(), should_exit, updated, dconf, emit)
+                turbo_start = True
                 first = False
-            continue
+            else:
+                continue
 
         try:
             logger.info("Launching emulated controller.")
             updated.clear()
             init = time.perf_counter()
-            controller_loop(conf.copy(), should_exit, updated, dconf, emit, turbo)
+            if turbo_start:
+                turbo_loop(conf.copy(), should_exit, updated, dconf, emit)
+            else:
+                controller_loop(conf.copy(), should_exit, updated, dconf, emit, turbo)
             repeated_fail = False
         except Exception as e:
             failed_fast = init + LONGER_ERROR_MARGIN > time.perf_counter()
@@ -123,6 +189,50 @@ def plugin_run(
 
     # Close the volume keyboard cache
     UInputDevice.close_volume_cached()
+    unhide_all()
+
+
+class OxpAtKbd(GenericGamepadEvdev):
+    def __init__(self, *args, **kwargs):
+        super().__init__(
+            *args,
+            **kwargs,
+        )
+        self.state = {}
+        self.queued = []
+
+    def produce(self, fds):
+        evs = list(super().produce(fds))
+        curr = time.perf_counter()
+        skip = []
+
+        for i, ev in enumerate(evs):
+            if ev["type"] == "button" and ev["code"] in (
+                "mode",
+                "keyboard",
+            ):
+                if ev["value"]:
+                    self.state[ev["code"]] = curr
+                else:
+                    t = self.state.pop(ev["code"], None)
+
+                    if t and curr - t < BUTTON_MIN_DELAY:
+                        self.queued.append((ev["code"], t + BUTTON_MIN_DELAY))
+                        skip.append(i)
+
+        for i in reversed(skip):
+            evs.pop(i)
+
+        while self.queued and self.queued[0][1] < curr:
+            evs.append(
+                {
+                    "type": "button",
+                    "code": self.queued.pop(0)[0],
+                    "value": False,
+                }  # type: ignore
+            )
+
+        return evs
 
 
 def find_vendor(prepare, turbo, protocol: str | None):
@@ -143,8 +253,17 @@ def find_vendor(prepare, turbo, protocol: str | None):
         turbo=turbo,
         required=True,
     )
+    d_hidraw_g1 = OxpHidraw(
+        vid=[XFLY_VID],
+        pid=[XFLY_PID],
+        usage_page=[XFLY_PAGE],
+        usage=[XFLY_USAGE],
+        turbo=turbo,
+        required=True,
+        g1=True,
+    )
 
-    if not protocol or protocol in ["serial", "mixed"]:
+    if protocol in ["serial", "mixed"]:
         try:
             prepare(d_ser)
             # OneXFly uses serial only for the buttons and hidraw for RGB
@@ -165,7 +284,7 @@ def find_vendor(prepare, turbo, protocol: str | None):
         except Exception as e:
             pass
 
-    if not protocol or protocol == "hid_v1":
+    if protocol == "hid_v1":
         try:
             prepare(d_hidraw)
             logger.info("Found OXP V1 hidraw vendor device.")
@@ -173,7 +292,15 @@ def find_vendor(prepare, turbo, protocol: str | None):
         except Exception as e:
             pass
 
-    if not protocol or protocol == "hid_v2":
+    if protocol == "hid_v1_g1":
+        try:
+            prepare(d_hidraw_g1)
+            logger.info("Found OXP V1 hidraw vendor device.")
+            return [d_hidraw_g1]
+        except Exception as e:
+            pass
+
+    if protocol == "hid_v2":
         try:
             prepare(d_hidraw_v2)
             logger.info("Found OXP V2 hidraw vendor device.")
@@ -198,7 +325,10 @@ def turbo_loop(
     if dconf.get("rgb_secondary", False):
         rgb_modes = RGB_MODES_FULL
     elif dconf.get("rgb", True):
-        rgb_modes = RGB_MODES_STICKS
+        if dconf.get("aok", False):
+            rgb_modes = RGB_MODES_STICKS_AOK
+        else:
+            rgb_modes = RGB_MODES_STICKS
     else:
         rgb_modes = None
 
@@ -211,7 +341,7 @@ def turbo_loop(
         controller_disabled=True,
     )
 
-    d_kbd_1 = GenericGamepadEvdev(
+    d_kbd_1 = OxpAtKbd(
         vid=[KBD_VID],
         pid=[KBD_PID],
         required=False,
@@ -226,16 +356,18 @@ def turbo_loop(
     qam_no_release = False
     if conf.get("turbo_reboots", False):
         share_reboots = True
-    match conf.get("extra_buttons", "separate"):
-        case "separate":
-            keyboard_is = "steam_qam"
-            qam_hhd = True
-        case "combo":
-            keyboard_is = "qam"
-            qam_hhd = False
-        case "combo_hhd":
-            keyboard_is = "qam"
-            qam_hhd = True
+
+    if not dconf.get("g1", False):
+        match conf.get("extra_buttons", "separate"):
+            case "separate":
+                keyboard_is = "steam_qam"
+                qam_hhd = True
+            case "combo":
+                keyboard_is = "qam"
+                qam_hhd = False
+            case "combo_hhd":
+                keyboard_is = "qam"
+                qam_hhd = True
 
     multiplexer = Multiplexer(
         trigger="analog_to_discrete",
@@ -252,7 +384,7 @@ def turbo_loop(
         keyboard_no_release=not conf.get("swap_face", False),
     )
 
-    if dconf.get("x1", False) and conf.get("volume_reverse", False):
+    if conf.get("volume_reverse", False):
         logger.info("Reversing volume buttons.")
         btn_map = {
             "key_volumedown": EC("KEY_VOLUMEUP"),
@@ -295,6 +427,7 @@ def turbo_loop(
     try:
         prepare(d_volume_btn)
         d_vend = find_vendor(prepare, True, dconf.get("protocol", None))
+        d_vend_id = [id(d) for d in d_vend]
 
         for d in d_producers:
             prepare(d)
@@ -309,7 +442,7 @@ def turbo_loop(
             if start - last_controller_check > TURBO_CONTROLLER_CHECK:
                 last_controller_check = start
                 try:
-                    found_device = bool(enumerate_evs(vid=GAMEPAD_VID))
+                    found_device = bool(enumerate_evs(vid=GAMEPAD_VID, pid=GAMEPAD_PID))
                 except Exception:
                     logger.warning("Failed finding device, skipping check.")
                     found_device = True
@@ -324,9 +457,13 @@ def turbo_loop(
                 to_run.add(id(fd_to_dev[f]))
 
             for d in devs:
-                if id(d) in to_run or d in d_vend:
+                d_id = id(d)
+                if d_id in to_run or d_id in d_vend_id:
                     evs.extend(d.produce(r))
 
+            # Read delayed events
+            evs.extend(d_kbd_1.produce([]))
+
             evs = multiplexer.process(evs)
             if evs:
                 if debug:
@@ -366,15 +503,24 @@ def controller_loop(
 ):
     debug = DEBUG_MODE
 
+    if dconf.get("rgb_secondary", False):
+        rgb_modes = RGB_MODES_FULL
+    elif dconf.get("rgb", True):
+        if dconf.get("aok", False):
+            rgb_modes = RGB_MODES_STICKS_AOK
+        else:
+            rgb_modes = RGB_MODES_STICKS
+    else:
+        rgb_modes = None
+
     # Output
     d_producers, d_outs, d_params = get_outputs(
         conf["controller_mode"],
         None,
         conf["imu"].to(bool),
         emit=emit,
-        rgb_modes=(
-            RGB_MODES_FULL if dconf.get("rgb_secondary", False) else RGB_MODES_STICKS  # type: ignore
-        ),
+        rgb_modes=rgb_modes,  # type: ignore
+        extra_buttons=dconf.get("extra_buttons", "dual"),
     )
     motion = d_params.get("uses_motion", True)
 
@@ -403,13 +549,23 @@ def controller_loop(
     else:
         mappings = BTN_MAPPINGS_NONTURBO
 
-    d_kbd_1 = GenericGamepadEvdev(
+    d_kbd_1 = OxpAtKbd(
         vid=[KBD_VID],
         pid=[KBD_PID],
         required=False,
         grab=True,
         btn_map=mappings,
     )
+    # Touchpad keyboard
+    d_kbd_2 = GenericGamepadEvdev(
+        vid=[0x6080],
+        pid=[0x8060],
+        required=True,
+        grab=False,
+        btn_map=BTN_MAPPINGS,
+        capabilities={EC("EV_KEY"): [EC("KEY_D")]},
+        requires_start=True,
+    )
 
     share_reboots = False
     keyboard_is = "keyboard"
@@ -446,7 +602,7 @@ def controller_loop(
         keyboard_no_release=not conf.get("swap_face", False),
     )
 
-    if dconf.get("x1", False) and conf.get("volume_reverse", False):
+    if conf.get("volume_reverse", False):
         logger.info("Reversing volume buttons.")
         btn_map = {
             "key_volumedown": EC("KEY_VOLUMEUP"),
@@ -491,6 +647,9 @@ def controller_loop(
 
     try:
         d_vend = find_vendor(prepare, turbo, dconf.get("protocol", None))
+        d_vend_id = [id(d) for d in d_vend]
+        if dconf.get("g1", False):
+            prepare(d_kbd_2)
         prepare(d_xinput)
         if motion:
             start_imu = True
@@ -515,9 +674,13 @@ def controller_loop(
                 to_run.add(id(fd_to_dev[f]))
 
             for d in devs:
-                if id(d) in to_run or d == d_vend:
+                d_id = id(d)
+                if d_id in to_run or d_id in d_vend_id:
                     evs.extend(d.produce(r))
 
+            # Read delayed events
+            evs.extend(d_kbd_1.produce([]))
+
             evs = multiplexer.process(evs)
             if evs:
                 if debug:
