@@ -175,6 +175,26 @@ class Config:
             return self[key].to(type(default))
         except KeyError:
             return default
+        except TypeError:
+            return default
+
+    def pop(self, key, default: A) -> A:
+        try:
+            val = self[key].to(type(default))
+            del self[key]
+            return val
+        except KeyError:
+            return default
+        except TypeError:
+            return default
+
+    def get_action(self, key):
+        if key not in self:
+            return False
+        tmp = bool(self[key].conf)
+        if tmp:
+            self[key] = False
+        return tmp
 
     def to(self, t: type[A]) -> A:
         return cast(t, self.conf)
